@@ -29,7 +29,7 @@
 from neuros import Neuros
 import neuros as neuros_module
 import util
-
+from os import path
     
 def run(config, neuros, args):
     if len(args) == 0:
@@ -42,7 +42,8 @@ def run(config, neuros, args):
             database.pack()
 
             if config.sort_database:
-                database.sort()
+                database.sort(path.join(*neuros.mountpoint_parts +
+                                        [neuros.DB_DIR, 'tracks.txt']))
                 
             neuros.close_db(arg)
         except neuros_module.Error, e:
